#lang racket
(require "util.rkt" rackunit)

(define input (input-lines 14))

(define (parse line)
  (let ([pattern #px"\\w+ can fly (\\d+) km/s for (\\d+) seconds, but then must rest for (\\d+) seconds."])
    (map string->number (rest (regexp-match pattern line)) )))

(define (make-flying sec)
  (λ (reindeer)
    (let-values ([(velocity flying-time resting-time) (apply values reindeer)])
      (for/fold ([time sec]
                 [distance 0]
                 #:result distance)
                ([v-t (in-cycle (in-list (list (list velocity flying-time)
                                               (list 0 resting-time))))]
                 #:break (negative? time))
        (values (- time (second v-t))
                (+ distance (apply * v-t)))) )))

(define (make-flying2 sec)
  (λ (reindeer)
    (let*-values ([(velocity flying-time resting-time) (apply values reindeer)]
                  [(flying-rest-cycles rest-time) (quotient/remainder sec (+ flying-time
                                                                             resting-time))])
      (+ (* flying-rest-cycles velocity flying-time)
         (* (if (< rest-time flying-time)
                rest-time
                flying-time) velocity)))))

(define flying-first (make-flying2 2503))

(define (solve-first lines)
  (apply max (map (compose1 flying-first parse) lines)))

(module+ test
  (define input (list
                 "Comet can fly 14 km/s for 10 seconds, but then must rest for 127 seconds."
                 "Dancer can fly 16 km/s for 11 seconds, but then must rest for 162 seconds."))
  (check-equal? (apply max (map (compose1 (make-flying2 1000) parse) input)) 1120)
  )
