#lang racket
(require "util.rkt" rackunit)
(define input-str (get-input file->string 3 "first"))
(define (next-house instruction current-house)
  (let ([instruction-ht (hash #\^ (list 0 1)
                              #\v (list 0 -1)
                              #\> (list 1 0)
                              #\< (list -1 0))])
    (map + current-house (hash-ref instruction-ht instruction))))


(define get-houses
  (lambda (str #:when [when number?])
    (define-values (n return)
      (for/fold ([n 0]
           [result (list (list 0 0))])
          ([ch (in-string str)])
    (values (+ 1 n) (if (when n)
                        (cons (next-house ch (first result)) result)
                        result))))
    return))

(define (solve-first str)
  (let ([all-houses (get-houses str)])
    (length (remove-duplicates all-houses))))

(module+ test
  (check-equal? (solve-first input-str) 2081)
  (check-equal? (solve-first ">") 2)
  (check-equal? (solve-first "^>v<") 4)
  (check-equal? (solve-first "^v^v^v^v^v") 2))

(define (solve-second str)
  (let ([santa-houses (get-houses str #:when even?)]
        [robo-houses (get-houses str #:when odd?)])
    (length (remove-duplicates (append santa-houses robo-houses)))))

(module+ test
  ;(check-equal? (solve-second input-str) 2081)
  (check-equal? (solve-second "^v") 3)
  (check-equal? (solve-second "^>v<") 3)
  (check-equal? (solve-second "^v^v^v^v^v") 11))
