#lang racket
(require rackunit "util.rkt")


(define input (get-input file->lines 5 "first"))
(define (make-regexp? pattern)
  (λ (str)
    (regexp-match? pattern str)))

(define three-vovewls?
  (λ (str)(let ([match (regexp-match* #rx"[uiaeo]" str)])
            (and match
                (> (length match) 2)))))

;; (define twice-in-row? (make-regexp? #px".{2}"))

(define (make-twice-with-n-between n)
  (λ (str)
    (for/first ([a (in-string str)]
                [b (in-string(substring str (add1 n)))]
                #:when (char=? a b))
      #t)))

(define twice-in-row? (make-twice-with-n-between 0))

(define ab-cd-pq-xy? (make-regexp? #rx"(ab|cd|pq|xy)"))


(define (pair-twice? str)
  (for/first ([a (in-string str)]
              [b (in-string (substring str 1))]
              [n (in-naturals 2)]
              #:when (regexp-match? (regexp (string a b))
                                   (substring str n)))
    #t))

(define twice-with-one-between? (make-twice-with-n-between 1))


(module+ test
  (check-equal? (twice-with-one-between? "xxyxx") #t)
  (check-equal? (pair-twice? "xxyxx") #t)
  (check-equal? (twice-with-one-between? "ieodomkazucvgmuy") #t)
  (check-equal? (pair-twice? "ieodomkazucvgmuy") #f)
  (check-equal? (twice-with-one-between? "uurcxstgmygtbstg") #f)
  (check-equal? (pair-twice? "uurcxstgmygtbstg") #t)
  (check-equal? (twice-with-one-between? "qjhvhtzxzqqjkmpb") #t)
  (check-equal? (pair-twice? "qjhvhtzxzqqjkmpb") #t))

(define (solve-first lines)
  (define (nice? str)
    (and
     (not (ab-cd-pq-xy? str))
     (twice-in-row? str)
     (three-vovewls? str)))
  (length (filter nice? lines)))

(define (solve-second lines)
  (define (nice? str)
    (and
     (twice-with-one-between? str)
     (pair-twice? str)))
  (length (filter nice? lines)))
(module+ test
  (check-equal? (solve-first input) 258))
