#lang racket
(require "util.rkt" rackunit)

(define (get-locations lines)
  (define with-duplicates (for/list ([line lines])
    (let* ([splitted (string-split line " ")]
           [src (first splitted)]
           [dest (third splitted)])
      (list src dest))))
  (remove-duplicates (flatten with-duplicates)))


(define (get-ht lines)
  (for/fold ([h (hash)])
            ([line lines])
    (let-values ([( _ city1 city2 distance) (apply values
                                                   (regexp-match #px"(\\w+) to (\\w+) = (\\d+)" line))])
      (hash-set* h (string-append city1 "->" city2) distance (string-append city2 "->" city1) distance))))

(define input (get-input file->lines 9 "first"))

(define (make-solver f)
  (define (solver lines)
  (define ht (get-ht lines))
  (define locations (get-locations lines))
  (define (->distance city1 city2) (string->number (hash-ref ht (string-append city1 "->" city2))))
  (let ([distance-permutations (for/list ([l (in-permutations locations)])
                                 (for/list ([city1 (take l (- (length l) 1))]
                                            [city2 (rest l)])
                                   (->distance city1 city2)))])
    (apply f (map ((curry apply) +) distance-permutations)))) solver)

(define solve-first (make-solver min))
(define solve-second (make-solver max))


(module+ test
  (define test-lines (list "London to Dublin = 464"
                           "London to Belfast = 518"
                           "Dublin to Belfast = 141"))
  (check-equal? (solve-first test-lines) 605)
  (check-equal? (solve-second test-lines) 982))
