#lang racket
(require rackunit "util.rkt")

(define adjacent-relative-coordinates (list '(-1 -1) '(0 -1) '(1 -1)
                                            '(-1  0)         '(1  0)
                                            '(-1  1) '(0  1) '(1  1)))

(define get-x first)
(define get-y second)

(define (get-field waiting-area x y)
  (list-ref (list-ref waiting-area y) x))

(define get-max-x (λ (waiting-area)((compose1 length first) waiting-area)))

(define get-max-y (λ (waiting-area)(length waiting-area)))

(define (coordinate-in-bound? waiting-area coordinate)
  (define max-x (get-max-x waiting-area))
  (define max-y (get-max-y waiting-area))
  (define make-in-bound? (λ (get max) (λ (coordinates)
                                        (let ([x (get coordinates)])
                                          (and (>= x 0)
                                               (< x max))))))
  (let ([x-in-bound? (make-in-bound? get-x max-x)]
        [y-in-bound? (make-in-bound? get-y max-y)])
    (and (x-in-bound? coordinate)
         (y-in-bound? coordinate))))

(define (make-adjacent-seat predicate)
  (λ (waiting-area x y)
    (define max-x (get-max-x waiting-area))
    (define max-y (get-max-y waiting-area))
    (let* ([make-coordinate-stream (λ (start direction)
                                     (let loop ([next (map + start direction)])
                                       (if (coordinate-in-bound? waiting-area next)
                                           (stream-cons next (loop (map + next direction)))
                                           empty-stream)))]
           [get (λ (coordinate) (get-field waiting-area
                                           (get-x coordinate)
                                           (get-y coordinate)))]
           [get-first-coordinate
            (λ (rel-coordinate)
              (for/first ([coordinate (in-stream (make-coordinate-stream (list x y)
                                                                         rel-coordinate))]
                          #:when (predicate (get coordinate)))
                (get coordinate)))])
      (filter-map get-first-coordinate adjacent-relative-coordinates))))

(define adjacent-seats (make-adjacent-seat (λ (_) #t)))

(define (seats-occupied fields)
  (apply + (map (λ (x) (if (equal? x #\#)
                           1
                           0))
                fields)))

(define (adjacent-seats-occupied waiting-area x y)
  (seats-occupied (adjacent-seats waiting-area x y)))

(define (make-round adjacent-seats-f occiupied->free)
  (λ (waiting-area)
    (for/list ([y (range (get-max-y waiting-area))])
      (for/list ([x (range (get-max-x waiting-area))])
        (let ([current (get-field waiting-area x y)])
          (if (equal? #\. current)
              #\.
              (let ([occupied (seats-occupied (adjacent-seats-f waiting-area x y))])
                (cond [(and (equal? #\L current)
                            (= occupied 0)) #\#]
                      [(and (equal? #\# current)
                            (>= occupied occiupied->free)) #\L]
                      [else current]))))))))

(define round (make-round adjacent-seats 4))

(define (make-solver round-f)
  (λ (waiting-area)
    (for/fold ([acc waiting-area]
               #:result (seats-occupied (flatten acc)))
              ([i (in-naturals)]
               #:break (equal? acc (round-f acc)))
      (round-f acc))))

(define solve-first (make-solver round))

(define adjacent-seats-no-walk-path (make-adjacent-seat (λ (seat)
                                                          (not (equal? seat #\.)))))

(define round2 (make-round adjacent-seats-no-walk-path 5))

(define solve-second (make-solver round2))


(define example-input (map string->list (list "L.LL.LL.LL" "LLLLLLL.LL" "L.L.L..L.." "LLLL.LL.LL" "L.LL.LL.LL"
                                              "L.LLLLL.LL" "..L.L....." "LLLLLLLLLL" "L.LLLLLL.L" "L.LLLLL.LL")))
(define example-input-1.-apply (map string->list (list "#.##.##.##" "#######.##" "#.#.#..#.." "####.##.##" "#.##.##.##"
                                                       "#.#####.##" "..#.#....." "##########" "#.######.#" "#.#####.##")))
(define example-input-2.-apply (map string->list (list "#.LL.L#.##" "#LLLLLL.L#" "L.L.L..L.." "#LLL.LL.L#" "#.LL.LL.LL"
                                                       "#.LLLL#.##" "..L.L....." "#LLLLLLLL#" "#.LLLLLL.L" "#.#LLLL.##")))
(define example-input-3.-apply (map string->list (list "#.##.L#.##" "#L###LL.L#" "L.#.#..#.." "#L##.##.L#" "#.##.LL.LL"
                                                       "#.###L#.##" "..#.#....." "#L######L#" "#.LL###L.L" "#.#L###.##")))
(define example-input-4.-apply (map string->list (list "#.#L.L#.##" "#LLL#LL.L#" "L.L.L..#.." "#LLL.##.L#" "#.LL.LL.LL"
                                                       "#.LL#L#.##" "..L.L....." "#L#LLLL#L#" "#.LLLLLL.L" "#.#L#L#.##")))
(define input (map string->list (input->lines 11)))

(define example-part2-8-seats-occiupied (map string->list (list ".......#." "...#....." ".#......." "........." "..#L....#"
                                                                "....#...." "........." "#........" "...#.....")))
(define example-part2-after-1.-apply (map string->list (list "#.##.##.##" "#######.##" "#.#.#..#.." "####.##.##" "#.##.##.##"
                                                             "#.#####.##" "..#.#....." "##########" "#.######.#" "#.#####.##")))
(module+ test
  (check-equal? (round example-input) example-input-1.-apply)
  (check-equal? (round example-input-1.-apply) example-input-2.-apply)
  (check-equal? (round example-input-2.-apply) example-input-3.-apply)
  (check-equal? (round example-input-3.-apply) example-input-4.-apply)
  (check-equal? (solve-first example-input) 37)
  (check-equal? (seats-occupied (adjacent-seats-no-walk-path example-part2-8-seats-occiupied 3 4))8)
  (check-equal? (round2 example-input) example-part2-after-1.-apply)
  (check-equal? (solve-second example-input) 26))
