#lang racket
(require rackunit "util.rkt")

(define (parse line)
  (let* ([pattern #px"(.)(\\d+)"]
         [match (regexp-match pattern line)])
    (list (second match) (string->number (third match)))))

(define (rotate direction-of-rotation degrees current-direction)
  (define direction-of-rotation-ht (hash "L" 1 "R" -1))
  (define directions (list "N" "W" "S" "E"))
  (define directions-len (length directions))
  (define take-direction (apply hash
                                (flatten (for/list ([v (range 4)]
                                                    [k directions])
                                           (list k v)))))
  (list-ref directions (modulo (+ (hash-ref take-direction current-direction)
                                  (* (hash-ref direction-of-rotation-ht direction-of-rotation)
                                     (/ degrees 90)))
                               directions-len)))

(define (manhatten lines)
  (for/fold ([east/west 0]
             [south/north 0]
             [dir "E"]
             #:result (+ (abs east/west)
                         (abs south/north)))
            ([line (in-list lines)])
    (let* ([p (parse line)]
           [arg (second p)]
           [instruction (let ([instruction (first p)])
                          (if (string=? instruction "F")
                              dir
                              instruction))])
      (match instruction
        ["N" (values east/west (+ south/north arg) dir)]
        ["S" (values east/west (- south/north arg) dir)]
        ["W" (values (+ east/west arg) south/north dir)]
        ["E" (values (- east/west arg) south/north dir)]
        [_ (values east/west south/north (rotate instruction arg dir))]))))

(define (rotate-instruction->counter-clockwise direction-of-rotation degrees)
  (let ([without-overdrive (modulo degrees 360)])
    (if (equal? direction-of-rotation "L")
        without-overdrive
        (- 360 without-overdrive))))

(define (rotate-waypoint waypoint α)
  (let  ([radians (* pi (cond [(= 90 α) 1/2]
                              [(= 180 α) 1]
                              [(= 270 α) 3/2]
                              [(= 360 α) 2]))])
    (make-polar (magnitude waypoint)
                (+ radians
                   (angle waypoint)))))

(define (manhatten2 lines)
  (for/fold ([waypoint 10+1i]
             [ship 0+0i]
             #:result (inexact->exact (+ (abs (imag-part ship))
                                         (abs (real-part ship))) )
             )
            ([line (in-list lines)])
    (let*-values ([(instruction arg) (apply values (parse line))]
                  [(add-ship) (λ (s) (values waypoint (+ ship s)))]
                  [(add-waypoint)(λ (w) (values (+ waypoint w) ship))]
                  [(rotate-waypoint) (λ ()
                                       (values (rotate-waypoint waypoint
                                                                (rotate-instruction->counter-clockwise instruction
                                                                                                       arg))
                                               ship))])
      (match instruction
        ["N" (add-waypoint (* arg 0+1i))]
        ["S" (add-waypoint (* arg 0-1i))]
        ["E" (add-waypoint arg)]
        ["W" (add-waypoint (- arg))]
        ["F" (add-ship (* waypoint arg))]
        [_ (rotate-waypoint)]))))
(define input (input->lines 12))

(module+ test
  (define eg-input (list "F10"
                         "N3"
                         "F7"
                         "R90"
                         "F11"))
  (check-equal? (manhatten eg-input) 25)
  (check-equal? (manhatten2 eg-input) 286))
