#lang racket
(require rackunit "util.rkt")

(define input (input-lines 3))

(define (geology-ref line n)
  (string-ref line (modulo (- n 1) (string-length line))))

(define (tree? line n)
  (equal? #\# (geology-ref line n)))

(define (traverse-trees right-step down-step lines)
  (define prepared-lines (for/list ([i (in-list (drop lines down-step))]
                                    [n (in-naturals)]
                                    #:when (= 0 (modulo n down-step)))
                           i))
  (for/fold ([right (+ right-step 1)]
             [trees 0]
             #:result trees)
            ([line (in-list prepared-lines)])
    (values (+ right right-step)
            (+ trees (if (tree? line right)
                         1
                         0)))))

(define (solve-first lines)
  (traverse-trees 3 1 lines))

(define (solve-second lines)
  (let* ([right-list (list 1 3 5 7 1)]
        [down-list (list 1 1 1 1 2)]
        [lines-list (make-list (length down-list) lines)])
    (apply * (map traverse-trees right-list down-list lines-list))))

(module+ test
  (define test-input (list "..##......."
                           "#...#...#.."
                           ".#....#..#."
                           "..#.#...#.#"
                           ".#...##..#."
                           "..#.##....."
                           ".#.#.#....#"
                           ".#........#"
                           "#.##...#..."
                           "#...##....#"
                           ".#..#...#.#"))
  (check-equal? (solve-second test-input) 336)
  (check-equal? (solve-first test-input) 7))
