#lang at-exp racket
(require rackunit "util.rkt")

(define input (input-lines 4))

(define (lines->passport-strings lines)
    (map (λ (strs)
           (string-join strs " "))
         (for/fold ([acc '()]
                    [subacc '()]
                    #:result (cons subacc acc))
                   ([line  (in-list lines)])
           (if (equal? line "")
               (values (cons subacc acc) '())
               (values acc (cons line subacc))))))

(define (passport-string->key-value-list passport-string)
  (let* ([pattern @pregexp{(\w+):([\w\d#]+)}]
         [replace-pattern @~a{\1 "\2"}]
         [list-str @~a{(@(regexp-replace* pattern passport-string replace-pattern))}])
    ((compose1 read open-input-string) list-str)))

(define (passport-string->hash str)
  (define key-value-list (passport-string->key-value-list str))
(apply hash key-value-list))

(define (passport-hts lines)
  (map passport-string->hash (lines->passport-strings  lines)))

(define (has-necessary-keys? ht)
  (let ([keys (list 'byr 'iyr 'eyr 'hgt 'hcl 'ecl 'pid)])
    (and (andmap (λ (key)
              (hash-has-key? ht key))
            keys) ht)))


(define (solve-first lines)
  (define hts (passport-hts lines))
    (length (filter has-necessary-keys? hts)))

(define (four-digit  x)
  (and (regexp-match? @pregexp{^\d\d\d\d$} x) (string->number x)))

(define (hair-color?  x)
  (regexp-match? #px"#[0-9a-f]{6}" x))

(define (height?  x)
  (regexp-match? #px"1([5-8]\\d|9[0-3])cm|(59|6\\d|7[0-6])in" x))

(define (eye-color?  x)
  (regexp-match? #rx"(amb)|(blu)|(brn)|(gry)|(grn)|(hzl)|(oth)" x))

(define (passport-id?  x)
  (regexp-match? #px"^\\d{9}$" x))

(define (within-years?  str lower upper)
  (let ([year (four-digit str)])
    (and year
         (>= year lower)
         (<= year upper))))

(define (valid? ht)
  (let-values ([(byr iyr eyr hgt hcl ecl pid)
                (apply values
                       (map ((curry hash-ref) ht)
                            (list 'byr 'iyr 'eyr 'hgt 'hcl 'ecl 'pid)))])
    (and
     (within-years? byr 1920 2002)
     (within-years? iyr 2010 2020)
     (within-years? eyr 2020 2030)
     (height? hgt)
     (hair-color? hcl)
     (eye-color? ecl)
     (passport-id? pid))))


(define (solve-second lines)
  (length (filter valid? (filter  has-necessary-keys? (passport-hts lines)) )))

(module+ test
(define test-input (list "ecl:gry pid:860033327 eyr:2020 hcl:#fffffd"
                         "byr:1937 iyr:2017 cid:147 hgt:183cm"
                         ""
                         "iyr:2013 ecl:amb cid:350 eyr:2023 pid:028048884"
                         "hcl:#cfa07d byr:1929"
                         ""
                         "hcl:#ae17e1 iyr:2013"
                         "eyr:2024"
                         "ecl:brn pid:760753108 byr:1931"
                         "hgt:179cm"
                         ""
                         "hcl:#cfa07d eyr:2025 pid:166559648"
                         "iyr:2011 ecl:brn hgt:59in"))
  (check-equal? (solve-first test-input) 2)

(define string->lines (compose1 passport-hts port->lines open-input-string))

(define valid-passwords
  (string->lines @~a{pid:087499704 hgt:74in ecl:grn iyr:2012 eyr:2030 byr:1980
                  hcl:#623a2f

                  eyr:2029 ecl:blu cid:129 byr:1989
                  iyr:2014 pid:896056539 hcl:#a97842 hgt:165cm

                  hcl:#888785
                  hgt:164cm byr:2001 iyr:2015 cid:88
                  pid:545766238 ecl:hzl
                  eyr:2022

                  iyr:2010 hgt:158cm hcl:#b6652a ecl:blu byr:1944 eyr:2021 pid:093154719}))

(define invalid-passwords
  (string->lines @~a{eyr:1972 cid:100
             hcl:#18171d ecl:amb hgt:170 pid:186cm iyr:2018 byr:1926

             iyr:2019
             hcl:#602927 eyr:1967 hgt:170cm
             ecl:grn pid:012533040 byr:1946

             hcl:dab227 iyr:2012
             ecl:brn hgt:182cm pid:021572410 eyr:2020 byr:1992 cid:277

             hgt:59cm ecl:zzz
             eyr:2038 hcl:74454a iyr:2023
             pid:3556412378 byr:2007})
  )
(check-equal? (andmap valid? valid-passwords) #t)
(check-equal? (ormap valid? invalid-passwords) #f)

(check-equal? (within-years? "2002" 1920 2002) #t)
(check-equal? (within-years? "2003" 1920 2002) #f)

(check-equal? (height? "60in") #t)
(check-equal? (height? "190cm") #t)
(check-equal? (height? "190in") #f)
(check-equal? (height? "190") #f)

(check-equal? (hair-color? "#123abc") #t)
(check-equal? (hair-color? "#123abz") #f)
(check-equal? (hair-color? "123abc") #f)

(check-equal? (eye-color? "amb") #t)
(check-equal? (eye-color? "blu") #t)
(check-equal? (eye-color? "brn") #t)
(check-equal? (eye-color? "gry") #t)
(check-equal? (eye-color? "grn") #t)
(check-equal? (eye-color? "hzl") #t)
(check-equal? (eye-color? "oth") #t)

(check-equal? (eye-color? "brn") #t)
(check-equal? (eye-color? "wat") #f)

(check-equal? (passport-id? "000000001") #t)
(check-equal? (passport-id? "0123456789") #f))
