#lang racket
(require rackunit "util.rkt"
         (for-syntax "util.rkt" racket/string))


(define  (get-bag-functions)
  (define input (input->lines 7))
  (define (bag-name line)
    (apply (λ (optical-prop color)
             (string-append optical-prop "-" color))
           (take (string-split line " ") 2)))
  (map (compose1 read open-input-string bag-name) input))

(define-syntax (convert-input-to-bag-functions stx)
  (let* ([input-strings (input->lines 7)]
        [remove-unnecessary-stuff (λ (line)
                                    (string-replace line #rx"bags?|,|\\." ""))]
        [replace-contain (λ (line) (string-replace line "contain" "->"))]
        [hyphen-bags (λ (line) (regexp-replace* #px"([a-z]+)\\s+([a-z]+)" line "\\1-\\2"))]
        [parenthise-args (λ (line) (regexp-replace* #px"(\\d+)\\s+(\\w+-\\w+)" line "(\\1 \\2)"))]
        [cleared-input-strings (map (compose1 parenthise-args hyphen-bags replace-contain remove-unnecessary-stuff)
                                    input-strings)]
        [bag-strings (map (λ (str) (format "(bag ~a)" str)) cleared-input-strings)]
        [bag-datums (map (compose1 read open-input-string) bag-strings)])
    (datum->syntax stx `(begin ,@bag-datums) )))

(define-for-syntax two-args->two-lists
  (λ args
    (for/fold ([count '()]
               [names '()]
               #:result (list count names))
              ([i (in-naturals)]
               [a (in-list args)])
      (if (even? i)
          (values (cons a count) names)
          (values count (cons a names))))))

(define-syntax (bag stx)
  (syntax-case stx (-> no-other)
    [(_ bag-name -> no-other)
     #'(define (bag-name n) (build-bag-list n (~a  'bag-name)))]
    [(_ bag-name -> (count inside-bag) ...)
     #'(define (bag-name n) (append (build-bag-list n (~a 'bag-name))
                                    (inside-bag count)...))]
    [(_) #'"not defined"]))

(define (build-bag-list n bag)
  (build-list n (λ (x) bag)))


(convert-input-to-bag-functions)
(define (solve-first)
  (length (filter (λ (x) (member "shiny-gold" x)) (map (λ (x) (read `(,x 1) )) (get-bag-functions)) )))
(solve-first)

  (define test-input (list "light red bags contain 1 bright white bag, 2 muted yellow bags."
                           "dark orange bags contain 3 bright white bags, 4 muted yellow bags."
                           "bright white bags contain 1 shiny gold bag."
                           "muted yellow bags contain 2 shiny gold bags, 9 faded blue bags."
                           "shiny gold bags contain 1 dark olive bag, 2 vibrant plum bags."
                           "dark olive bags contain 3 faded blue bags, 4 dotted black bags."
                           "vibrant plum bags contain 5 faded blue bags, 6 dotted black bags."
                           "faded blue bags contain no other bags."
                           "dotted black bags contain no other bags."))
(module+ test)
