#lang racket
(require "util.rkt" rackunit racket/generator racket/stream)

(define input (input->lines 8))


(define (parse line)
  (let-values ([(cmd arg) (apply values (string-split line " "))])
    (values cmd (string->number arg))))


(define (terminates lines)
  (define program (if (list? lines)
                      (list->vector lines)
                      lines))
  (define program-length (vector-length program))
  (for/fold ([pc 0]
             [acc 0]
             [allready-called '()]
             #:result (list (= pc program-length) acc))
            ([_ (in-naturals)]
             #:break (or (member pc allready-called)
                         (= pc program-length)))
    (let*-values ([(cmd arg)(parse (vector-ref program pc))]
                  [(cmd? )(λ (x) (string=? cmd x))]
                  [(new-allready-called) (cons pc allready-called)])
      (cond [(cmd? "nop") (values (add1 pc) acc new-allready-called)]
            [(cmd? "acc") (values (add1 pc) (+ acc arg) new-allready-called)]
            [(cmd? "jmp") (values (+ pc arg) acc new-allready-called)]))))


(define (solve-first lines)
  (match-define  (list terminates? acc) (terminates lines))
  acc)

(define (generate-mutated-programs lines)
  (define program (list->vector lines))
  (define program-length (vector-length program))
  (define (swap line)
    (match-let ([(list i a)(string-split line " ")])
      (cond [(string=? i "nop") (string-append "jmp " a)]
          [(string=? i "jmp")(string-append "nop " a)]
          [else #f])))
  (define (new-program? index)
  (define current-instruction (vector-ref program index))
    (define swapped (swap current-instruction))
    (if swapped
        (vector-append (vector-take program index)
                         (vector swapped)
                         (vector-drop program (add1 index)))
        #f))
  (filter-map new-program? (range program-length)))

(define (solve-second lines)
  (for/first ([program (in-list (generate-mutated-programs lines))]
              #:when (first (terminates program)))
    (second (terminates program))))

  (define test-lines (list "nop +0"
                           "acc +1"
                           "jmp +4"
                           "acc +3"
                           "jmp -3"
                           "acc -99"
                           "acc +1"
                           "jmp -4"
                           "acc +6"))

(module+ test
  (define test-lines (list "nop +0"
                           "acc +1"
                           "jmp +4"
                           "acc +3"
                           "jmp -3"
                           "acc -99"
                           "acc +1"
                           "jmp -4"
                           "acc +6"))

  (check-equal? (solve-first test-lines) 5)
  (define infinite-lines (list "jmp +0"
                               "acc +1"
                               "jmp +4"
                               "acc +3"
                               "jmp -3"
                               "acc -99"
                               "acc +1"
                               "jmp -4"
                               "acc +6"))

  (define terminates-lines (list "nop +0"
                                 "acc +1"
                                 "jmp +4"
                                 "acc +3"
                                 "jmp -3"
                                 "acc -99"
                                 "acc +1"
                                 "nop -4"
                                 "acc +6"))
(check-equal? (terminates terminates-lines) (list #t 8))
(check-equal? (terminates infinite-lines) (list #f 0))
  (check-equal? (solve-second test-lines) 8)
  )
