#lang racket
(require rackunit "util.rkt")

(define (sums-of-previous prev)
  (for/fold ([p prev]
             [acc '()]
             #:result (flatten acc))
            ([_ prev])
    (values (rest p)
            (cons (for/list ([i (rest p)])
                    (+ i (first p)))
                  acc))))

(define (not-in-sums-of-previous? n prev)
  (if (member n (sums-of-previous prev))
      #f
      n))

(define (first/not-in-previous n lines)
  (for/fold ([prev (take lines n)]
             [not-in-previous #f]
             #:result  not-in-previous)
            ([i (in-list (drop lines n))]
             #:break not-in-previous)
    (values (append (rest prev) (list i))
            (not-in-sums-of-previous? i prev))))

(define input (map string->number (input->lines 9)))

(define (solve-first lines)
  (first/not-in-previous 25 lines))

(define (contigous-set n lines)
  (define (drop-unless-sum<=n acc)
    (if (<= (apply + acc) n)
        acc
        (drop-unless-sum<=n (rest acc))))
  (for/fold ([acc '()])
            ([i (in-list lines)]
             #:break (= (apply + acc) n))
    (let ([new-acc (append acc (list i))])
      (if (> (apply + new-acc) n)
          (drop-unless-sum<=n new-acc)
          new-acc))))

(define (solve-second lines n)
  (let* ([first-invalid (first/not-in-previous n lines)]
         [c-set (contigous-set first-invalid lines)])
    (+ (apply min c-set)
       (apply max c-set))))

(define test-input (map string->number (list "35"
                                             "20"
                                             "15"
                                             "25"
                                             "47"
                                             "40"
                                             "62"
                                             "55"
                                             "65"
                                             "95"
                                             "102"
                                             "117"
                                             "150"
                                             "182"
                                             "127"
                                             "219"
                                             "299"
                                             "277"
                                             "309"
                                             "576")))
(module+ test
  (check-equal? (first/not-in-previous 5 test-input) 127)
  (check-equal? (solve-second  test-input 5) 62))
